;;; side-note.el --- Visual and durable side note annotations    -*- lexical-binding: t -*-

;; Copyright (C) 2020 Free Software Foundation, Inc

;; Author: Michael Heerdegen <michael_heerdegen@web.de>
;; Maintainer: Michael Heerdegen <michael_heerdegen@web.de>
;; Created: 08 Jan 2020
;; Keywords: k1, k2
;; Compatibility: GNU Emacs ??
;; Version: 0.0.1
;; Package-Requires: ((emacs "25") (cl-lib "0"))


;; This file is not part of GNU Emacs.

;; GNU Emacs is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.


;;; Commentary:

;; Side notes for Emacs.
;; TODO: A lot.

;;; Code:

;;;; Requirements

(eval-when-compile
  (require 'eieio)
  (require 'cl-lib))


;;;; Configuration stuff

;;;; Helpers and Definitions

(defvar-local side-note-list '()
  "List of buffer side notes.")

(defclass side-note () ((text :initarg :text) ;i.e. content
                        (remove))) ;function to remove visualization

(cl-defstruct side-note-save-location
  "Holding type and data of a buffer's save location for side notes."
  tag data)

(cl-defgeneric side-note-list-notes-in-location (location)
  "Create an occur-like overview for side notes saved in side-note-save-location LOCATION.")

(defvar-local side-note-save-location nil ;must be set explicitly
  "Describes place of storing side notes."
  ;; Should multiple places be supported?
  )

(cl-defgeneric side-note-visualize (side-note)
  "Visualize SIDE-NOTE.
How this is done can depend on note type and the major-mode.
Should return a function to remove the visualization.")

(cl-defgeneric side-note-get-save-form (note)
  "Return a form suitable for saving the SIDE-NOTE."
  note)

(cl-defgeneric side-note-save-in-location (location)
  "Save current buffer's side-notes in LOCATION.")


;;;; Main Stuff

(defun side-note-save ()
  (interactive)
  (side-note-save-in-location side-note-save-location))

(defun side-note-add (note)
  (oset note remove (side-note-visualize note))
  (push note side-note-list)
  (if buffer-file-name
      (set-buffer-modified-p t)
    (side-note-save)))

(defun side-note-remove (note)
  (funcall (oref note remove))
  (cl-callf2 delq note side-note-list))

(defun side-note-load-notes (&optional buffer)
  ;; Look for side notes for BUFFER (defaulting to the current buffer)
  ;; in all implemented locations, in some defined order; setup
  ;; `side-note-save-location' and `side-note-list', and display the
  ;; side notes
  )

(define-minor-mode side-note-mode
  "Minor mode for creation of durable side nodes."
  ;;of course there will be a useful keymap...
  )

(define-globalized-minor-mode global-side-note-mode side-note-mode side-note-load-notes)


;;; Simplest type: Position side notes

(defclass side-note-pos-note (side-note)
  ((pos :initarg :pos)))

(defun side-note-add-note-at-pos (pos text)
  (interactive (list (point) (read-string "Note text: ")))
  (side-note-add
   (side-note-pos-note
    :pos (copy-marker pos)
    :text text)))

(cl-defmethod side-note-visualize ((note side-note-pos-note))
  (let* ((pos (oref note pos))
         (ov (make-overlay pos (1+ pos))))
    (overlay-put ov 'face 'highlight)
    (overlay-put ov 'help-echo (oref note text))
    (lambda () (delete-overlay ov))))


(provide 'side-note)
;;; side-note.el ends here
